/*
  Copy EEPROM to microSD SDEEPROM

*/
#include "ubitx.h"
#include <ili9341_t3n_font_Arial.h>
#include <ili9341_t3n_font_ArialBold.h>
#include <ILI9341_t3n.h>
#include <SPI.h>
#include <SD.h>
#include <EEPROM.h>


extern void tftPrint(int x, int y, char* Text, ILI9341_t3_font_t, int fontColor, bool textframed, int frameColor );

extern int   Working_Copy_Modes[10];
extern unsigned long Working_Copy_Frequencies[10];
extern void setFrequency(unsigned long xf);

extern ILI9341_t3n tft ;


extern int CAL_Minx;
extern int CAL_Miny;
extern int CAL_Maxx;
extern int CAL_Maxy;
extern char c[21];
extern char vfoActive;
extern unsigned long frequency;
extern int Current_Band;
extern unsigned long vfoA, vfoB;

unsigned long VFO_A_Band_Table[8];
unsigned long VFO_B_Band_Table[8];

File fileptr;

int TableMODES[4][10] = { 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
                          1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
                          0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                          1, 1, 2, 1, 2, 1, 1, 1, 1, 1
                        };


unsigned long TableFREQS[4][10] = { 3560000, 7030000, 7122000, 10106000, 10116000, 14060000, 18096000, 21060000, 24906000, 28060000,
                                    3985000, 7285000, 7090000, 14285000, 18130000, 21285000, 21385000, 24950000, 28365000, 28385000,
                                    3580000, 3576000, 7070000, 7076000, 3562000, 7277500, 7280000, 7280000, 7280000, 7280000,
                                    7277000, 3920000, 3984000, 3547000, 7290000, 3562000, 7277500, 7280000, 7280000, 7280000
                                  };


unsigned long BandTableFREQS[8] = { 3560000, 7030000, 10106000, 14060000, 18096000, 21060000, 24906000, 28060000 };



void SD_OPERATION_FAILED(void) {

  tft.fillScreen(ILI9341_WHITE);
  tft.setFont(Arial_18_Bold);
  tft.setTextColor(ILI9341_RED);
  tft.setCursor(12, 140);
  tft.print((char *)"SD DISK FAIL->EEPROM");
  while ( 1 == 1) delay(100);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
void Read_Band_Table_from_SD(void) {
  File fileptr;
  char fred[14];



  if ( vfoActive == VFO_A ) fileptr = SD.open("VFOABTBL", FILE_WRITE);
  else fileptr = SD.open("VFOBBTBL", FILE_WRITE);
  if ( fileptr ) {
    fileptr.seek( Current_Band * 8 );
    fileptr.read( fred ,8 );
    frequency = atol( fred );
  if ( vfoActive == VFO_A ) vfoA = frequency;
  else vfoB = frequency;
    setFrequency(frequency);
    fileptr.close();
  } else SD_OPERATION_FAILED();
}



void Write_Band_Table_Update_to_SD( void ) {
  char fred[20];


  if ( vfoActive == VFO_A ) fileptr = SD.open("VFOABTBL", FILE_WRITE);
  else fileptr = SD.open("VFOBBTBL", FILE_WRITE);
  if ( fileptr ) {
    fileptr.seek( Current_Band * 8 );
    sprintf(fred, "%08d", (int)frequency);
    fileptr.write( fred );
    fileptr.close();
  } else SD_OPERATION_FAILED();
  Read_Band_Table_from_SD();


}



void Write_Entire_Band_Table_to_SD(void) {
  char fred[20];
  unsigned long i;
  File VFOA_fileptr;
  File VFOB_fileptr;



  // delete file
  SD.remove("VFOABTBL");
  SD.remove("VFOBBTBL");
  // open file SDEEPROM for writing
  VFOA_fileptr = SD.open("VFOABTBL", FILE_WRITE);
  if ( VFOA_fileptr ) {
    for ( i = 0; i < 8; i++) {
      VFOA_fileptr.seek(i * 8);
      sprintf(fred, "%08d", (int)BandTableFREQS[i]);
      VFOA_fileptr.write( fred );
    }
    VFOA_fileptr.close();
  } else SD_OPERATION_FAILED();
  VFOB_fileptr = SD.open("VFOBBTBL", FILE_WRITE);
  if ( VFOB_fileptr ) {
    for ( i = 0; i < 8; i++) {
      VFOB_fileptr.seek(i * 8);
      sprintf(fred, "%08d", (int)BandTableFREQS[i]);
      VFOB_fileptr.write( fred, 8 );
    }
    VFOB_fileptr.close();
  } else SD_OPERATION_FAILED();
  Read_Band_Table_from_SD();
}






bool Put_SDEEPROM( int index, long Value ) {
  byte Data[4];

  // open file SDEEPROM for writing
  fileptr = SD.open("SDEEPROM", FILE_WRITE);
  if ( fileptr ) {
    // move position to beginning of file
    fileptr.seek(index);
    Data[3] = Value & 0xff;
    Data[2] = (Value >> 8) & 0xff;
    Data[1] = (Value >> 16) & 0xff;
    Data[0] = (Value >> 24) & 0xff;
    fileptr.write(Data, 4);
    fileptr.close();
  } else SD_OPERATION_FAILED();
  return (true);

}




bool Backup_EEPROM_to_SD( void ) {
  int i;
  long temp;

  for (i = 0; i < 76; i = i + 4) {
    EEPROM.get(i, temp);
    if ( !Put_SDEEPROM(i, temp)) return (false);
  }


  return ( true );

}


long Get_SDEEPROM( int index ) {
  File fileptr;
  byte Data[4];
  long Value;

  // open file SDEEPROM for writing
  fileptr = SD.open("SDEEPROM", FILE_READ);
  if ( fileptr ) {
    fileptr.seek(index);
    fileptr.read(Data, 4);

    Value = Data[0] & 0xff;
    Value = (Value << 8) + Data[1];
    Value = (Value << 8) + Data[2];
    Value = (Value << 8) + Data[3];
    fileptr.close();

  } else {
    SD_OPERATION_FAILED();
  }
  return (Value);
}

extern int minx, miny, maxx, maxy;
extern   File fileptr;

void Write_Screen_CAL_to_SD( void ) {
  char fred[7];

  sprintf(fred, "%04d\n", minx);
  Serial.println(fred);
  fileptr.write(fred);

  sprintf(fred, "%04d\n", maxx);
  Serial.println(fred);
  fileptr.write(fred);

  sprintf(fred, "%04d\n", miny);
  fileptr.write(fred);

  sprintf(fred, "%04d\n", maxy);
  fileptr.write(fred);
}


void Read_Screen_CAL_to_SD( void ) {
  char fred[5];

  fileptr.read(fred, 4);
  fred[4] = 0;
  CAL_Minx = atoi(fred);

  fileptr.read(fred, 4);
  fred[4] = 0;
  CAL_Maxx = atoi(fred);

  fileptr.read(fred, 4);
  fred[4] = 0;
  CAL_Miny = atoi(fred);

  fileptr.read(fred, 4);
  fred[4] = 0;
  CAL_Maxy = atoi(fred);


}

void Write_Default_Memory_Tables( void ) {
  int i;
  for (i = 0; i < 10; i++) {
    Put_SDEEPROM( 100 + (i * 4), TableFREQS[0][i] );
    Put_SDEEPROM( 140 + (i * 4), TableMODES[0][i] );
  }
  for (i = 0; i < 10; i++) {
    Put_SDEEPROM( 200 + (i * 4), TableFREQS[1][i] );
    Put_SDEEPROM( 240 + (i * 4), TableMODES[1][i] );
  }
  for (i = 0; i < 10; i++) {
    Put_SDEEPROM( 300 + (i * 4), TableFREQS[2][i] );
    Put_SDEEPROM( 340 + (i * 4), TableMODES[2][i] );
  }
}

void Write_Default_USR_Table( void ) {
  int i;

  for (i = 0; i < 10; i++) {
    Put_SDEEPROM( 400 + (i * 4), TableFREQS[3][i] );
    Put_SDEEPROM( 440 + (i * 4), TableMODES[3][i] );
  }
}


void Do_SD_Init( void ) {

  if (!Backup_EEPROM_to_SD() ) {
    SD_OPERATION_FAILED();
  } else {
    Write_Default_Memory_Tables();
  }

}


void SD_Setup() {
  int i;


  if (!SD.begin(BUILTIN_SDCARD)) {
    SD_OPERATION_FAILED();
  } else {
    // open file SDEEPROM for writing
    fileptr = SD.open("SDEEPROM", FILE_WRITE);
    if ( fileptr ) {
      i = fileptr.size();
      if (i < 50 )Do_SD_Init();
      fileptr.close();
    } else {
      Do_SD_Init();
    }
  }
}

